package oracle.spatial.raster;

import oracle.spatial.hadoop.imageprocessor.datatype.ImageBandWritable;
import oracle.spatial.hadoop.imageprocessor.process.ImageProcessorInterface;

import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.IntWritable;

import org.gdal.gdal.gdal;


public class HillShade implements ImageProcessorInterface {
    public HillShade() {
        super();
    }

    @Override
    public ImageBandWritable process(ImageBandWritable imageBandWritable) throws Exception {
        //the GeneralInfoWritable inside the image contains the information about the final mosaic output, get number of output bands from there.
        int bands = Integer.valueOf(imageBandWritable.getMosaicInfo().getBandsToAdd().toString().split(",").length);

        int width = imageBandWritable.getDstWidthSize().get();
        int height = imageBandWritable.getDstHeightSize().get();

        int overlap = imageBandWritable.getOverlap().get();
        byte upOv = imageBandWritable.getUpOv().get();
        byte downOv = imageBandWritable.getDownOv().get();
        byte leftOv = imageBandWritable.getLeftOv().get();
        byte rightOv = imageBandWritable.getRightOv().get();

        int startX = 0;
        int startY = 0;
        int endX = width;
        int endY = height;
        //If there are overlapping pixels on the left, then X will start in position overlap otherwise in 1
        if (leftOv == Byte.MAX_VALUE) {
            startX = overlap;
            imageBandWritable.setOffX(new IntWritable(imageBandWritable.getOffX().get() + overlap));
        } else {
            startX = 1;
            imageBandWritable.setOffX(new IntWritable(imageBandWritable.getOffX().get() + 1));
        }
        //If there are overlapping pixels on the right, then X will end in the width position minus overlapping pixels
        if (rightOv == Byte.MAX_VALUE) {
            endX = width - overlap;
        }
        //If there are overlapping pixels on the top, then Y will start in position overlap
        if (upOv == Byte.MAX_VALUE) {
            startY = overlap;
            imageBandWritable.setOffY(new IntWritable(imageBandWritable.getOffY().get() + overlap));
        }
        //If there are overlapping pixels on the bottom, then Y will end in the height position minus overlapping pixels
        if (downOv == Byte.MAX_VALUE) {
            endY = height - overlap;
        }

        hillshadeTile(imageBandWritable, startX, startY, endX, endY, width, height, bands);
        return imageBandWritable;

    }


    private void hillshadeTile(ImageBandWritable image, int startX, int startY, int endX, int endY, int width,
                               int height, int bands) {
        HillShadeUtil util = new HillShadeUtil();

        int lastValue = width * height;
                                double pixelXWidth = image.getMosaicInfo().getPixelXWidth().get();
                                double pixelYWidth = image.getMosaicInfo().getPixelYWidth().get();
                                int newWidth = endX - startX;
                                int newHeight = endY - startY;
                    
                                image.resetBandCounter();
                                
                                
        for (int x = 1; x <= bands; x++) {
                                                DoubleWritable noData = image.getNoDataArray(x);
                                util.hillshade(lastValue, pixelXWidth, pixelYWidth, noData, image, newWidth, newHeight, x, startX,
                                                                                          startY, endX, endY, width, height);
        }


        image.setDstWidthSize(new IntWritable(newWidth));
                                image.setDstHeightSize(new IntWritable(newHeight));
                                int pixels = (newWidth) * (newHeight);
                                int bytesNumber = pixels * (gdal.GetDataTypeSize(image.getDType().get()) / 8);
                                image.setBytesNumber(bytesNumber);
        DoubleWritable[] originalGeoTransform = image.getGeoTransform();
        DoubleWritable[] adfGeoTransform = new DoubleWritable[6];
        //0 and 3rd positions represent image origin, recalculating for new image size.
        adfGeoTransform[0] =
            new DoubleWritable(originalGeoTransform[0].get() + (originalGeoTransform[1].get() * startX));
        adfGeoTransform[1] = new DoubleWritable(originalGeoTransform[1].get());
        adfGeoTransform[2] = new DoubleWritable(originalGeoTransform[2].get());
        adfGeoTransform[3] =
            new DoubleWritable(originalGeoTransform[3].get() + (originalGeoTransform[5].get() * startY));
        adfGeoTransform[4] = new DoubleWritable(originalGeoTransform[4].get());
        adfGeoTransform[5] = new DoubleWritable(originalGeoTransform[5].get());
        image.setGeoTransform(adfGeoTransform);


    }


}
