package oracle.raster.test;

import oracle.spatial.hadoop.imageprocessor.util.ProcessConstants;
import oracle.spatial.hadoop.rasterapi.core.HadoopConfiguration;
import oracle.spatial.hadoop.rasterapi.core.beans.MosaicConfiguration;
import oracle.spatial.hadoop.rasterapi.core.beans.Raster;
import oracle.spatial.hadoop.rasterapi.core.beans.RasterCatalog;
import oracle.spatial.hadoop.rasterapi.core.jobs.RasterLoaderJob;
import oracle.spatial.hadoop.rasterapi.core.jobs.RasterProcessorJob;

public class RasterElevationExample {
    public RasterElevationExample() {
        super();
    }
    
    public static void main(String args[]){
        HadoopConfiguration hadoopConf = new HadoopConfiguration();
        String gdalData = "/opt/oracle/oracle-spatial-graph/spatial/raster/gdal/data";
        hadoopConf.setGdalDataPath(gdalData);
        hadoopConf.setMapreduceJobJar("hadoop-imageloader.jar");
        String rasterDirectory = "/opt/oracle/oracle-spatial-graph/spatial/raster/Raster-HOL/data/raster/NapaDEM.tif";

                                        RasterLoaderJob loader = (RasterLoaderJob) hadoopConf.createRasterLoaderJob();
        loader.setFilesToLoad(rasterDirectory);
        loader.setOutputFolder("exercise5");
        
        loader.setRasterThumbnailFolder( "/opt/shareddir/spatial");
        loader.setTileOverlap("30");
        String gdalDirectory = "/opt/oracle/oracle-spatial-graph/spatial/raster/gdal/lib";      
        try {
                        loader.setGdalPath(gdalDirectory);
        //Executes the job
        boolean loaderSuccess = loader.execute();

        if (loaderSuccess) {
         System.out.println("Successfully executed loader job");
            hadoopConf = new HadoopConfiguration();
                     hadoopConf.setGdalDataPath(gdalData);
                       hadoopConf.setMapreduceJobJar("hadoop-imageprocessor.jar");
            RasterProcessorJob processor = (RasterProcessorJob) hadoopConf.createRasterProcessorJob();
            processor.setGdalPath(gdalDirectory);
            MosaicConfiguration mosaic = new MosaicConfiguration();
            mosaic.setBands(1);
            mosaic.setDirectory("/opt/shareddir/spatial");
            mosaic.setFileName("APIMosaicSlope");
            mosaic.setFileSystem(RasterProcessorJob.FS);
            mosaic.setFormat("GTIFF");
            mosaic.setSrid("32610");
            mosaic.setWidth(1400);            
            mosaic.setHeight(870);
                            //float32 datatype
                      mosaic.setPixelType("6");
                      mosaic.setNoData("#00");
                       //in case two or more rasters overlap, the area covered by the rasters will determine its priority in the mosaic
                      mosaic.setOrderAlgorithm(ProcessConstants.ALGORITHM_FILE_LENGTH);
                      //rasters that cover less area will be located on top of the mosaic, DESC, descending order
                      mosaic.setOrder(RasterProcessorJob.DESC);
               
                             //width for pixels in X and Y
                      mosaic.setPixelXWidth(48.416923);
                      mosaic.setPixelYWidth(-48.416923);
                      //upper left coordinates
                      mosaic.setUpperLeftX(520284.694956);
                      mosaic.setUpperLeftY(4305151.843027);
            mosaic.setProcessingClasses("oracle.spatial.hadoop.imageprocessor.process.ImageSlope");
            RasterCatalog catalog = new RasterCatalog();

            //Creates a raster object for the catalog
            Raster raster = new Raster();
            raster.setBands(1);
            raster.setBandsOrder("1");
            raster.setDataType(6);
            raster.setRasterLocation("/user/hdfs/exercise5/opt/oracle/oracle-spatial-graph/spatial/raster/Raster-HOL/data/raster/NapaDEM.tif.ohif");
            //Add raster to catalog
            catalog.addRasterToCatalog(raster);
            processor.setMosaicConfigurationObject(mosaic.getCompactMosaic());
            processor.setCatalogObject(catalog.getCompactCatalog());

            boolean processorSuccess = processor.execute();
                     if (processorSuccess) {
                         System.out.println("Successfully executed processor job");
                     } else {
                         System.out.println("Failed to execute processor job");
                     }

         
        } else {
         System.out.println("Failed to execute loader job");
        }
                       } catch (Exception e) {
                    System.out.println("Problem when trying to execute raster loader " + e.getMessage());
                }


    }
}
