package hol;

import java.util.Map;

import oracle.spatial.geometry.JGeometry;
import oracle.spatial.hadoop.vector.RecordInfoProvider;
import oracle.spatial.hadoop.vector.util.JsonUtils;

import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;

/**
 * RecordInfoProvider implementation that parse a GeoJSON text. Additionally to provide geometry it adds
 * the followers_count property to the extra fields list.
 *
 */
public class TweetsRecordInfoProvider implements RecordInfoProvider<LongWritable, Text>{
	private ObjectMapper jsonMapper = null;
	private JsonNode recordNode = null;
	/**
	 * Creates a new instance
	 */
	public TweetsRecordInfoProvider(){
		//create a unique json mapper to parse all the records
		jsonMapper = new ObjectMapper();
	}
	
	@Override
	public void setCurrentRecord(LongWritable key, Text value) {
		try{
			//parse the current value
			recordNode = jsonMapper.readTree(value.toString());
		}catch(Exception ex){
			ex.printStackTrace();
		}
	}

	@Override
	public String getId() {
		//returns the id of the record
		return recordNode.get("_id").getTextValue();
	}
	
	@Override
	public JGeometry getGeometry() {
		//returns the geometry of the record
		return JsonUtils.readGeometry(
						recordNode.get("geometry"), //the GeoJSON geometry
						2, //the dimensions of the geometry
						8307 //the SRID of the geometry
						);
	}
	@Override
	public boolean getExtraFields(Map<String, String> extraFields) {
		//Find the followers count
		String followersCount = recordNode.get("properties").get("followers_count").getValueAsText();
		//and add it to the extra fields that will be available when performing an analysis
		extraFields.put("followers_count", followersCount);

		//return true since we added an extra field
		return true;
	}
}
